/*
Copyright (C) 2003 by Sean David Fleming

sean@ivec.org

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The GNU GPL can also be found at http://www.gnu.org
*/

gpointer graph_new(const gchar *, struct model_pak *);
void graph_free_list(struct model_pak *);
void graph_free(gpointer, struct model_pak *);
void graph_add_data(gint, gdouble *, gdouble, gdouble, gpointer);
void graph_set_grafted(gint, gpointer);
void graph_set_xticks(gint, gint, gpointer);
void graph_set_yticks(gint, gint, gpointer);
void graph_set_wavelength(gdouble, gpointer);
void graph_set_select(gdouble, gchar *, gpointer);
void graph_write(gchar *, gpointer);
void graph_read(gchar *);
gchar *graph_treename(gpointer);

gint graph_grafted(gpointer);
gint graph_xlabel(gpointer);
gint graph_ylabel(gpointer);
gdouble graph_xmin(gpointer);
gdouble graph_xmax(gpointer);
gdouble graph_ymin(gpointer);
gdouble graph_ymax(gpointer);
gdouble graph_wavelength(gpointer);

/* TODO - relocate */
gint anim_next_frame(struct model_pak *);

